﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.ServiceModel.Configuration;

namespace TestSvrSample
{
    class Program
    {
        static void Main(string[] args)
        {
            using (ServiceHost host = new ServiceHost(typeof(TestSvr)))
            {
                host.Open();

                // 客户端调用
                using (ChannelFactory<ITest> factory = new ChannelFactory<ITest>("test_ep"))
                {
                    ITest ch = factory.CreateChannel();
                    try
                    {
                        int r = ch.Add(-1, 6);
                        Console.WriteLine(r);
                    }
                    catch(FaultException faultex)
                    {
                        FaultReason reason = faultex.Reason;
                        string text = reason.GetMatchingTranslation().Text;
                        Console.WriteLine($"错误：{text}");
                    }
                }

                Console.Read();
            }
        }
    }


    [ServiceContract(Namespace = "sp-test", Name = "demo", ConfigurationName = "testContract")]
    public interface ITest
    {
        [OperationContract]
        int Add(int a, int b);
        [OperationContract]
        int SQR(int n);
    }

    [ServiceBehavior(ConfigurationName = "sv")]
    class TestSvr : ITest
    {
        public int Add(int a, int b)
        {
            if (a < 0) throw new ArgumentException("值不能小于0", nameof(a));
            if (b < 0) throw new ArgumentException("值不能小于0", nameof(b));
            return a + b;
        }

        public int SQR(int n)
        {
            if (n == 0) throw new Exception("参数不能为0");
            return n * n;
        }
    }

    #region 自定义的错误处理
    public class CustErrorHandler : IErrorHandler
    {
        public bool HandleError(Exception error)
        {
            return true;
        }

        public void ProvideFault(Exception error, MessageVersion version, ref Message fault)
        {
            // 1、封装异常
            string errText = error.Message;
            FaultException fex = new FaultException(errText);
            // 2、产生 MessageFault
            MessageFault msg_fault = fex.CreateMessageFault();
            // 3、产生新的消息
            fault = Message.CreateMessage(version, msg_fault, "cust-fault");
        }
    }
    #endregion

    #region 扩展
    public class CustEndpointBehavior : IEndpointBehavior
    {
        public void AddBindingParameters(ServiceEndpoint endpoint, BindingParameterCollection bindingParameters)
        {
            return;
        }

        public void ApplyClientBehavior(ServiceEndpoint endpoint, ClientRuntime clientRuntime)
        {
            return; //无需处理
        }

        public void ApplyDispatchBehavior(ServiceEndpoint endpoint, EndpointDispatcher endpointDispatcher)
        {
            ChannelDispatcher cndisp = endpointDispatcher.ChannelDispatcher;
            // 加入自定义的错误处理程序
            CustErrorHandler cehdlr = null;
            cehdlr = (CustErrorHandler)cndisp.ErrorHandlers.FirstOrDefault();
            if (cehdlr == null)
            {
                cehdlr = new CustErrorHandler();
            }
            cndisp.ErrorHandlers.Add(cehdlr);
        }

        public void Validate(ServiceEndpoint endpoint)
        {
            return;
        }
    }

    public class CustBehaviorExtElement : BehaviorExtensionElement
    {
        public override Type BehaviorType => typeof(CustEndpointBehavior);

        protected override object CreateBehavior()
        {
            return new CustEndpointBehavior();
        }
    }
    #endregion
}
